/* --------------------------------------------------------------
    Tausworth3 random generator.

    Original code from Glenn Rhoads' Home Page
    http://remus.rutgers.edu/~rhoads/Code/code.html

    Adapted by Dr. Raul N.Shakirov, IMach of RAS(UB).

    Permission has been granted to copy, distribute and modify
    software in any context without fee, including a commercial
    application.

    THE SOFTWARE IS DISTRIBUTED "AS IS". NO WARRANTY OF ANY KIND
    IS EXPRESSED OR IMPLIED. YOU USE AT YOUR OWN RISK. THE AUTHOR
    WILL NOT BE LIABLE FOR DATA LOSS, DAMAGES, LOSS OF PROFITS OR
    ANY OTHER KIND OF LOSS WHILE USING OR MISUSING THIS SOFTWARE.
-------------------------------------------------------------- */
#include <limits.h>
#include <stdlib.h>
#include <time.h>
#include "Random3.h"

#ifdef  _MSC_VER
#pragma auto_inline (off)
#endif/*_MSC_VER*/

/* --------------------------------------------------------------
    Combination of 3 tausworth generators.
-------------------------------------------------------------- */

static unsigned long s1=390451501UL, s2=613566701UL, s3=858993401UL;
                                            /* The seeds */
static unsigned long mask1, mask2, mask3;
static long shft1, shft2, shft3;
static long k1=31, k2=29, k3=28;

/* use either of the following two sets of parameters*/
static long q1=13, q2=2, q3=3,  p1=12, p2=4,  p3=17;
/*
static long q1=3,  q2=2, q3=13, p1=20, p2=16, p3=7;
*/

#ifdef  __cplusplus
extern "C" {
#endif/*__cplusplus*/

/* --------------------------------------------------------------
    Set random seed.
-------------------------------------------------------------- */

void    SRand3 (unsigned long a, unsigned long b, unsigned long c)
{
    static unsigned long x = 4294967295UL;

    shft1=k1-p1;
    shft2=k2-p2;
    shft3=k3-p3;
    mask1 = x << (32-k1);
    mask2 = x << (32-k2);
    mask3 = x << (32-k3);
    if (a > (1u<<shft1)) s1 = a;
    if (b > (1u<<shft2)) s2 = b;
    if (c > (1u<<shft3)) s3 = c;

    lRand3();
}

/* --------------------------------------------------------------
    Set arbitrary random seed
-------------------------------------------------------------- */

void    RRand3 (void)
{
   /* Initialize Microsoft random generator */
   srand ((unsigned) time (NULL));

   /* Initialize tausworth3 random generator */
   SRand3 ((unsigned long) rand() * (unsigned long) rand(),
           (unsigned long) rand() * (unsigned long) rand(),
           (unsigned long) rand() * (unsigned long) rand());
}

/* --------------------------------------------------------------
    Generate random value in a range 0..ULONG_MAX.
-------------------------------------------------------------- */

unsigned long lRand3 (void)
{
   unsigned long temp = s1 ^ s2 ^ s3;

   s1 = ((s1 & mask1) << p1) ^ (((s1 << q1)^s1) >> shft1);
   s2 = ((s2 & mask2) << p2) ^ (((s2 << q2)^s2) >> shft2);
   s3 = ((s3 & mask3) << p3) ^ (((s3 << q3)^s3) >> shft3);

   return (temp);
}

/* --------------------------------------------------------------
    Generate random value in a range [0,1).
-------------------------------------------------------------- */

double  dRand3 (void)
{
    return ((double) lRand3() / ((double) ULONG_MAX + 1));
}

#ifdef  __cplusplus
}
#endif/*__cplusplus*/
